package edu.berkeley.sbp.meta;
import edu.berkeley.sbp.util.*;
import edu.berkeley.sbp.*;
import edu.berkeley.sbp.chr.*;
import edu.berkeley.sbp.misc.*;
import edu.berkeley.sbp.bind.*;
import java.util.*;
import java.lang.annotation.*;
import java.lang.reflect.*;
import java.io.*;

public class MetaGrammar {
    
    public static void main(String[] args) throws Exception {
        if (args.length != 2) {
            System.err.println("usage: java " + MetaGrammar.class.getName() + " grammarfile.g com.yourdomain.package.ClassName");
            System.exit(-1);
        }
        String className   = args[1].substring(args[1].lastIndexOf('.')+1);
        String packageName = args[1].substring(0, args[1].lastIndexOf('.'));
        String fileName    = packageName.replace('.', '/') + "/" + className + ".java";

        BufferedReader br = new BufferedReader(new InputStreamReader(new FileInputStream(fileName)));
        StringBuffer out = new StringBuffer();

        boolean skip = false;
        for(String s = br.readLine(); s != null; s = br.readLine()) {
            if (s.indexOf("DO NOT EDIT STUFF BELOW: IT IS AUTOMATICALLY GENERATED") != -1 && s.indexOf("\"")==-1) skip = true;
            if (s.indexOf("DO NOT EDIT STUFF ABOVE: IT IS AUTOMATICALLY GENERATED") != -1 && s.indexOf("\"")==-1) break;
            if (!skip) out.append(s+"\n");
        }

        out.append("\n        // DO NOT EDIT STUFF BELOW: IT IS AUTOMATICALLY GENERATED\n");

        Tree t = MetaGrammarTree.meta;
        Union u = MetaGrammar.make(t, "s");

        System.err.println();
        System.err.println("== parsing with parsed grammar =================================================================================");
        t = new CharParser((Union)u).parse(new FileInputStream(args[0])).expand1();
        System.out.println(t.toPrettyString());

        t.toJava(out);
        out.append("\n        // DO NOT EDIT STUFF ABOVE: IT IS AUTOMATICALLY GENERATED\n");

        for(String s = br.readLine(); s != null; s = br.readLine()) out.append(s+"\n");
        br.close();

        OutputStream os = new FileOutputStream(fileName);
        PrintWriter p = new PrintWriter(new OutputStreamWriter(os));
        p.println(out.toString());
        p.flush();
        os.close();
    }

    public static Union make() { return make(MetaGrammarTree.meta, "s"); }
    public static Union make(Tree t, String s) { return make(t, s, new AnnotationGrammarBindingResolver(MetaGrammarBindings.class)); }
    public static Union make(Tree t, String s, GrammarBindingResolver rm) {
        Tree.TreeFunctor<Object,Object> red = (Tree.TreeFunctor<Object,Object>)t.head();
        MetaGrammarBindings.GrammarNode g = (MetaGrammarBindings.GrammarNode)red.invoke(t.children());
        return g.build(s, rm);
    }

}
